package Network.Algorithm;

import Network.Representation.*;
import Network.Algorithm.*;

import java.util.Vector;

/** 
 * An abstract class to provide general functionalities required by all algorithms.
 * Algorithms should generally subclass this class. Once an instance of an algorithm created,
 * in order to be able to use it, the created algorithm must be directed to a particular network using <code>setTargetNetwork</code> method.
 * Directing an algorithm to a network results in making necesaary initializations to be made at
 * the network to which it is directed such as creating necessary information holding fields in
 * arcs and nodes. An algorithm may bi redirected to another network after it has been directed
 * to another network. While redirecting an algorithm to another network, there are two choices:
 * we may either choose not to clear the information fields regarding to this algorithm or we may
 * totally clear all the information that has been written on nodes and arcs by the algorithm which is
 * redirected. If, however, we choose not to clear the information in the old network, then accessing the
 * arc and node information on old network becomes tricky: we must keep in the algorithm order of the
 * algorithm on the old network and access the algorithm information fields of nodes and arcs via this
 * index.
 */

public abstract class AlgorithmImplementation implements Algorithm,Cloneable {
    
    public static ArcAlgInfoFactory arcInfoFactory = new ArcAlgInfoFactory(500, 500);
    public static NodeAlgInfoFactory nodeInfoFactory = new NodeAlgInfoFactory(500, 500);
    
    
    protected Network targetNetwork;
    protected int algorithmOrder;
    
    public AlgorithmImplementation(){
        targetNetwork=null;
    		//{{INIT_CONTROLS
		//}}
}
    
    public abstract NodeAlgInfo getInstanceOfNodeAlgInfo();
    
    public abstract ArcAlgInfo getInstanceOfArcAlgInfo();
    
    
    /**
     * Sets the target network for this algorithm without cleaning the old target network for this algorithm.
     *
     * @param   setNetwork  new target networh for this algorithm
     */
    public void setTargetNetwork(Network setNetwork) {
        setTargetNetwork(setNetwork,false);
    }
    
    
    /**
     * Sets the target network for this algorithm.
     *
     * @param   setNetwork    new target network for this algorithm
     * @param   removeOldInfo   clears the old information fields for this algorithm on the old target network
     * if this parameter is set true. Otherwise it leaves the information that has been previously written by this
     * algorithm on its old target network.
     */
    
    public void setTargetNetwork(Network setNetwork,boolean removeOldAlgInfo){
        
        if (targetNetwork!=null) {
            if (removeOldAlgInfo) {
                initializeInfo(true, false);
                getTargetNetwork().getAlgorithmList().put(getAlgorithmOrder()-1,null);
                getTargetNetwork().setNoOfAlgorithms(targetNetwork.getNoOfAlgorithms()-1);
            }
            else {
                try {
                    getTargetNetwork().getAlgorithmList().put(getAlgorithmOrder()-1,this.clone());
                }
                catch (CloneNotSupportedException e) {}
            }
        }
            
        COM.objectspace.jgl.Array algorithmList=setNetwork.getAlgorithmList();
        targetNetwork=setNetwork;
        int location;
        int n=algorithmList.size();
        boolean foundEmptySpace=false;
        for (location=0; location<n; location++) {
            if (algorithmList.at(location)==null) {
                foundEmptySpace=true;
                break;
            }
        }
        
        boolean add;
        if (foundEmptySpace) {
            algorithmOrder=location+1;
            add = false;
        }
        else {
            algorithmOrder=algorithmList.size()+1;
            add = true;
        }
        int capacity=Math.max(getAlgorithmOrder(), algorithmList.capacity());
        algorithmList.setSize(capacity);
        algorithmList.put(algorithmOrder-1,this);
        setNetwork.setNoOfAlgorithms(targetNetwork.getNoOfAlgorithms()+1);
        initializeInfo(false, add);
        
    }
    
    
    public final Network getTargetNetwork(){
        return targetNetwork;
    }
    
    
    
    /**
     * This method should normally NOT be used by users. This method is used when this algorithm is redirected
     * and old information that has been transcribed has to be cleaned.
     *
     * @param   clean   if true, arc info is collected and set to null
     * @param   add     if true, new arc info is added at end of Vector, else put at existing index
     */
     
    protected final void initializeInfo(boolean clean, boolean add){               
        NodeList nodeCursor=targetNetwork.getNodeList();
        ArcList arcCursor;
        
        Node myNode;
        Arc myArc;
        NodeAlgInfo nodeInfo;
        ArcAlgInfo arcInfo;
        int putIndex = getAlgorithmOrder() - 1;
        while (nodeCursor != null) {                      /*traverse the nodes and initialize*/
            myNode=nodeCursor.elem;
            if (!clean){
                nodeInfo=getInstanceOfNodeAlgInfo();
                Vector myNodeInfo = myNode.getNodeAlgorithmInfo();
                if ( (add) || (putIndex == myNodeInfo.size()) )
                    myNodeInfo.addElement(nodeInfo);
                else
                    myNodeInfo.setElementAt(nodeInfo, putIndex);
            }
            else {
                Vector myNodeInfo = myNode.getNodeAlgorithmInfo();
                AlgorithmImplementation.nodeInfoFactory.collectInfo((NodeAlgInfo)myNodeInfo.elementAt(putIndex));
                myNodeInfo.setElementAt(null, putIndex);
            }
            
            arcCursor=myNode.getOutBoundArcs();             /*traverse the out bound arcs of each node and initialize*/
            while (arcCursor!=null) {
                myArc=arcCursor.elem;
                if (!clean) {
                    arcInfo=getInstanceOfArcAlgInfo();
                    Vector myArcInfo = myArc.getArcAlgorithmInfo();
                    if ( (add) || (putIndex == myArcInfo.size()) )
                        myArcInfo.addElement(arcInfo);
                    else
                        myArcInfo.setElementAt(arcInfo, putIndex);
                }
                else {
                    Vector myArcInfo = myArc.getArcAlgorithmInfo();
                    AlgorithmImplementation.arcInfoFactory.collectInfo((ArcAlgInfo)myArcInfo.elementAt(putIndex));
                    myArcInfo.setElementAt(null, putIndex);
                }
                arcCursor=arcCursor.next;
            }
            
            nodeCursor=nodeCursor.next;
            
        }
    }
    
    
    
    
    
    
    
    
    
    /**
     * The place where this algorithm's information is kept in the <code>Array</code>  
     * <code>nodeAlgorithmInformation</code> (for <code>NodeImplementationClass</code>).
     */
    public final int getAlgorithmOrder(){
        return algorithmOrder;
    }
    
    /**
     * Returns a node's algorithm_information corresponding to this algorithm with generic type
     * <code>NodeAlgInfo</code>.
     * @param   whichNodesInfo  the node whose algorithm information is to be returned.
     */
    public final NodeAlgInfo getGeneralAlgNodeInfo(Node whichNodesInfo){
        return ((NodeAlgInfo)(whichNodesInfo.getNodeAlgorithmInfo().elementAt(algorithmOrder-1)));
    }

    /**
     * Returns an arc's algorithm_information corresponding to this algorithm with generic type
     * <code>ArcAlgInfo</code>.
     * @param   whichArcsInfo  the arc whose algorithm information is to be returned.
     */
    public final ArcAlgInfo getGeneralAlgArcInfo(Arc whichArcsInfo){
        return ((ArcAlgInfo) (whichArcsInfo.getArcAlgorithmInfo().elementAt(algorithmOrder-1)));
    }
    
    /**
     * Returns a string representation of this object with algorithm order and <code>targetNetwork.toString()</code> sepetated by spaces.
     */
    public String toString() {
        return Integer.toString(algorithmOrder)+" "+targetNetwork.toString();
    }
	//{{DECLARE_CONTROLS
	//}}
}