package Network.Representation;

import Network.Representation.*;

/**
 * This class in fact simulates a "retailer" rather than a "factory", because if it runs out of
 * arcs, it returns null rather than producing a new arc. In this way we were able to code "arc factories"
 * rather than "arc implementation factories". So users can store anything that implements <code>Arc</code> in
 * this factory and retrieve it when necessary.
 */
 
public class ArcFactory {
    
    private ArcStack storedArcs;
    private Node anchorNode;
    
    /**
     * Creates an arc factory with given initial size and size increment.
     */
    public ArcFactory (int initialSize, int sizeIncrement) {
        storedArcs=new ArcStack (initialSize, sizeIncrement);
        anchorNode=new NodeImplementation();
    }
    
    /** 
     * Get an arc from the inventory of arc factory. If the factory has run out arcs, this method returns null. Then
     * the user is free to produce any kind of arc he wants instead of getting a default arc class that would otherwise
     * be used by the factory.
     *
     * @param   arcCost the cost of the arc to be given by the arc factory.
     * @param   arcUpperBound   the upper boound of the arc to be supplied by the arc factory
     */
     
    public Arc makeArc(double arcCost, double arcUpperBound ){
        Arc dummyArc;
        dummyArc=storedArcs.pop();
        if ( dummyArc!=null ) {
            dummyArc.initializeArc(arcCost, arcUpperBound);
        }
        return (dummyArc);
    }
    
    
    /**
     * Puts the given arc ino the inventory of the arc factory. This arc should not belong to any network and should
     * not point to any other nodes. Otherwise <code>ArcError</code> will be thrown.
     */
     
    public void collectArc(Arc collectedArc) {
        storedArcs.push(collectedArc,anchorNode);
    }
    
    public String toString() {
        return storedArcs.toString();
    }

        
}





class ArcStack {
    
    Arc[] elements;
    int capacityIncrement;
    int noOfArcsStored;
    
    
    ArcStack (int size, int sizeIncrement) {
        elements=new Arc[size];
        capacityIncrement=sizeIncrement;
        noOfArcsStored=0;
    }
    
    final synchronized void push (Arc pushedArc, Node flagNode){
        if ( (pushedArc.getHeadNode()==null) && (pushedArc.getTailNode()==null) ) {
            
            if ( noOfArcsStored >= elements.length ) {
                resizeElements();
            }
            elements[noOfArcsStored]=pushedArc;
            pushedArc.setHeadNode(flagNode);
            pushedArc.setTailNode(flagNode);
            noOfArcsStored++;
        }
        else {
            throw new ArcError("arc can't be pushed into arc factory: arc already belongs to an object");
        }
    }
    
    
    
    final synchronized Arc pop() {
        
        if (noOfArcsStored > 0 ) {
            Arc dummyArc=elements[--noOfArcsStored];
            dummyArc.setHeadNode(null);
            dummyArc.setTailNode(null);
            return dummyArc;
        }
        else {
            return null;
        }
    }
    
    final synchronized boolean isEmpty() {
        return (noOfArcsStored==0);
    }
        
    
    final synchronized void resizeElements(){
        Arc[] newElements=new Arc[elements.length+capacityIncrement];
        System.arraycopy (elements, 0, newElements, 0, elements.length);
        elements=newElements;
    }
    
    public String toString() {
        return Integer.toString(noOfArcsStored)+" "+Integer.toString(elements.length)+" "+Integer.toString(capacityIncrement);
    }
    
    
}    

