package Network.Representation;

import Network.Algorithm.*;
import Network.Representation.*;

import java.util.*;

/**
 * A default implementation for interface <code>Node</code>.
 */

public class NodeImplementation implements Node {
    
    public static int defaultInfoSize = 10;
    
    int noOfInBoundArcs = 0;
    int noOfOutBoundArcs = 0;
    Network containerNetwork = null;
    ArcList inBoundArcs = null;
    ArcList outBoundArcs = null;
    double supply;
    Vector nodeAlgorithmInfo;
    
    /* Here! */
    private static final ArcListFactory arcListFactory = new ArcListFactory(5000, 2500);

    /**
     * Creates a node with the given supply. Created nodes should be added to a network with relevant
     * network's <code>addNode</code> method.
     * @see Network.Representation.NetworkImplementation#addNode
     */
    public NodeImplementation(int nodeSupply){
        supply=nodeSupply;
        nodeAlgorithmInfo=new Vector(defaultInfoSize);
        nodeAlgorithmInfo.setSize(0);
    }

    /**
     *Creates a node with 0 supply.
     */
    public NodeImplementation(){
        this(0);
    }


    /**
     * Initializes the node so that it has the given supply. The Node to be initialized should NOT beling to any
     * network otherwise <code>NodeError</code> will be thrown.
     */

    public final void initializeNode(int nodeSupply){
        if ( this.containerNetwork==null ) {
            supply=nodeSupply;
            cleanInBoundArcs();
            cleanOutBoundArcs(null);
            containerNetwork=null;
            nodeAlgorithmInfo.setSize(0);
        }
        else {
            throw new NodeError("can't initialize node: node belongs to a network");
        }
    }


    public final void addInBoundArc(Arc inBoundArc)
    {
        ArcList add = arcListFactory.make(inBoundArc);
        if (inBoundArcs != null)
            inBoundArcs.addAfter(add);
        else
            inBoundArcs = add;
        noOfInBoundArcs++;
    }

    public final void removeInBoundArc(Arc inBoundArc)
    {
        ArcList remove = inBoundArcs.remove(inBoundArc);
        if (inBoundArcs == remove)
            inBoundArcs = remove.next;
        noOfInBoundArcs--;
        remove.elem = null; //Helps GC
        arcListFactory.unmake(remove);
    }

    public final void addOutBoundArc(Arc outBoundArc)
    {
        ArcList add = arcListFactory.make(outBoundArc);
        if (outBoundArcs != null)
            outBoundArcs.addAfter(add);
        else
            outBoundArcs = add;
        noOfOutBoundArcs++;
    }

    public final void removeOutBoundArc(Arc outBoundArc)
    {
        ArcList remove = outBoundArcs.remove(outBoundArc);
        if (outBoundArcs == remove)
            outBoundArcs = remove.next;
        noOfOutBoundArcs--;
        remove.elem = null; //Helps GC
        arcListFactory.unmake(remove);
    }

    //Doesn't return any arcs to factory---this is done
    //in cleanOutBoundArcs of arc.getTailNode().  This is
    //so to avoid double-collecting the same arc.
    public final void cleanInBoundArcs()
    {
        ArcList clean = inBoundArcs;
        while (clean != null) {
            ArcList next = clean.next;
            clean.elem = null; //Helps GC
            arcListFactory.unmake(clean);
            clean = next;
        }
        inBoundArcs = null;
        noOfInBoundArcs = 0;
    }

    //Collects ArcList.elem if arcFactory != null, else relies
    //on GC.  As long as this method is invoked on every node,
    //every factory-made arc will be collected because every
    //arc must have a tail node from which it is outbound.  This method
    //also returns each arc's ArcAlgInfo to the AlgorithmImplementation.arcInfoFactory.
    public final void cleanOutBoundArcs(ArcFactory arcFactory)
    {
        ArcList clean = outBoundArcs;
        while (clean != null) {
            ArcList next = clean.next;
            clean.elem.cleanArcAlgorithmInfo();
            if (arcFactory != null) {
                clean.elem.setTailNode(null);
                clean.elem.setHeadNode(null);
                arcFactory.collectArc(clean.elem);
            }
            else
                clean.elem = null; //Helps GC
            arcListFactory.unmake(clean);
            clean = next;
        }
        outBoundArcs = null;
        noOfOutBoundArcs = 0;
    }

    /**
     * Returns the number of inbound arcs to this node.
     */
    public final int getNoOfInBoundArcs(){
        return noOfInBoundArcs;
    }

    /**
     * Users should normally NEVER use this method.
     */

    public void setNoOfInBoundArcs(int setNumber){
        noOfInBoundArcs=setNumber;
    }

    /**
     * Returns the supply of this node.
     */
    public final double getSupply(){
        return supply;
    }


    /**
     * Users should normally NEVER use this method.
     */
    public void setSupply(double mySupply){
        supply=mySupply;
    }

    public final int getNoOfOutBoundArcs(){
        return noOfOutBoundArcs;
    }

    /**
     * Users should normally NEVER use this method.
     */
    public void setNoOfOutBoundArcs(int setNumber ){
        noOfOutBoundArcs=setNumber;
    }

    /**
     * Returns the network to which this node belings.
     */
    public final Network getContainerNetwork(){
        return containerNetwork;
    }

    /**
     * Returns an <code>SList</code> enumeration of the inbound arcs to this node.
     */
    public final ArcList getInBoundArcs(){
        return inBoundArcs;
    }

    /**
     * Returns an <code>SList</code> enumeration of the outbound arcs from this node.
     */
    public final ArcList getOutBoundArcs(){
        return outBoundArcs;
    }

    /**
     * Returns an <code>Array</code> algorithm_information fields that have been transcribed into this node.
     */
    public final Vector getNodeAlgorithmInfo(){
        return nodeAlgorithmInfo;
    }

    public final void cleanNodeAlgorithmInfo(){
        for (int i = nodeAlgorithmInfo.size(); i-- > 0; ) {
            AlgorithmImplementation.nodeInfoFactory.collectInfo((NodeAlgInfo)nodeAlgorithmInfo.elementAt(i));
        }
        nodeAlgorithmInfo.setSize(0);
    }

    /**
     * Users should normally NEVER use this method.
     */
    public void setContainerNetwork(Network setNetwork){
        containerNetwork=setNetwork;
    }

    /**
     * Returns a <code>String</code> representation of this class consisting of "supply", "number of inbound arcs",
     * "number of outbound arcs" all seperated by spaces.
     */
    public String toString() {
        return Double.toString(supply)+" "+Integer.toString(noOfInBoundArcs)+" "+Integer.toString(noOfOutBoundArcs);
    }

}

